/* convmac.c	(C) 1992 taka@exsys.prm.prug.or.jp
	convert kanji-code for MAC */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <dirent.h>
#include "defs.h"

#define	JIS	1
#define	SJIS	2

main(argc,argv)
int	argc;
char	**argv;
{
	if (argc < 2) {
		usage(argv[0]);
		exit(0);
	}
	convmac(argc,argv);
	exit(0);
}

/* print usage */
usage(prg)
char	*prg;
{
	fprintf(stderr, "Usage: %s address ...\n",prg);
	fprintf(stderr, "ex. %s toshi@jf2krs.hpcc.prug.or.jp\n", prg);
}

/* display list */
convmac(argc,argv)
int	argc;
char	**argv;
{
char	buf[LINELEN];
char	buf1[3][LINELEN];
DIR	*dp;
struct dirent	*d;
FILE    *fp;
char	*p;
int	i;

	if ((dp = opendir(MQUEUE_DIR)) == NULL) {
		fprintf(stderr, "Can't open mqueue\n");
		exit(0);
	}
	/* countup */
	while((d = readdir(dp)) != NULL) {
		/* skip '.' /  '..' */
		if (d->d_name[0] == '.')
			continue;
		if ((p = strchr(d->d_name, '.')) == NULL)
			continue;
		if (strcmp(p+1, "wrk") != 0)
			continue;
		/* skip lck file */
		sprintf(buf,"%s/%s", MQUEUE_DIR, d->d_name);
		p = strchr(buf, '.');
		strcpy(p+1, "lck");
		if ((fp = fopen(buf, "r")) != NULL) {
			fclose(fp);
			continue;
		}
		/* open wrk file */
		sprintf(buf,"%s/%s", MQUEUE_DIR, d->d_name);
		if ((fp = fopen(buf,"r")) == NULL) {
			fprintf(stderr,"%s open error\n", buf);
			exit(0);
		}
		/* reading */
		for(i = 0; i < 3; i++) {
			if (fgets(buf1[i],LINELEN,fp) == NULL) {
				fprintf(stderr,"%s format error\n",buf);
				exit(0);
			}
			rip(buf1[i]);
		}
		fclose(fp);
		/* check From address */
		for(i = 1; i < argc; i++) {
			if (strcmp(argv[i], buf1[1]) == 0) {
				/* convert SJIS -> JIS */
				conv_exec(d->d_name,JIS);
				break;
			}
		}
		/* check to address */
		for(i = 1; i < argc; i++) {
			if (strcmp(argv[i], buf1[2]) == 0) {
				/* convert JIS->SJIS */
				conv_exec(d->d_name,SJIS);
				break;
			}
		}

	}
	closedir(dp);
}

/* strip cr/lf */
rip(s)
char *s;
{
	while(*s) {
		if (*s == '\r' || *s == '\n')
			*s = 0;
		++s;
	}
}

conv_exec(name,mode)
char	*name;
int	mode;
{
char	buf[LINELEN];
char	*p;
FILE	*in;
FILE	*out;
int	type;
	/* open txt file */
	sprintf(buf, "%s/%s", MQUEUE_DIR, name);
	p = strchr(buf, '.');
	strcpy(p+1,"txt");
	if ((in = fopen(buf, "r")) == NULL)
		return(-1);
	/* check kanji-code */
	type = check_code(in);
	if (mode == JIS && type == JIS) {
		fclose(in);
		return(0);
	}
	if (mode == SJIS && type == SJIS) {
		fclose(in);
		return(0);
	}
	fseek(in, 0,0);
	/* create temp file */
	out = tmpfile();
	/* convert */
	if (mode == JIS)
		sjis_jis(in, out);
	else
		jis_sjis(in, out);
	fclose(in);
	/* copy file */
	fseek(out, 0,0);
	in = fopen(buf, "w");
	while(fgets(buf,LINELEN, out) != NULL)
		fputs(buf, in);
	fclose(in);
	fclose(out);
}

#define ESC '\033'
#define KI1 '$' 
#define KI2 'B'
#define KI3 '@'
#define KO1 '('
#define KO2 'J'
#define KO3 'H'
#define KO4 'B'
#define KANAIN  0x0e
#define KANAOUT 0x0f

int check_code(in)
FILE	*in;
{
char	buf[LINELEN];
char	*p;
	while(fgets(buf, LINELEN, in) != NULL) {
		p = buf;
		while((p = strchr(p, ESC)) != NULL) {
			++p;
			if (*p == KI1 || *p == KO1)
				return(JIS);
		}
	}
	return(SJIS);
}

sjis_jis(INPUT, OUTPUT)
FILE	*INPUT;
FILE	*OUTPUT;
{
	register int stat;
	register unsigned c,c1,c2;
	unsigned s1,s2;

	stat = 0;
	while ((c = getc(INPUT)) != EOF) {
	   switch (stat) {
	   case 0:
		if((c>=0x81 && c<=0x9f) || (c>=0xe0 && c<=0xfc)) {
			c1 = c;	           
			stat = 1;
			putc(ESC,OUTPUT);	 /* put KANJI-IN */
			putc(KI1,OUTPUT);
			putc(KI2,OUTPUT);
		}
		else if((0xa1 <= c) && (c <= 0xdf)) { /* hankaku kana */
			putc(KANAIN,OUTPUT);
			c &= 0x7f;
			putc(c,OUTPUT);
			stat = 3;
		}
		else
			putc(c,OUTPUT);		 /* put ASCII code */
		break;
	   case 1:
		stat = 2;
		c2 = c;
		SJIS_JIS(c1,c2,&s1,&s2);  /* change shift-JIS into JIS */
		putc(s1,OUTPUT);	  /* put KANJI code */
		putc(s2,OUTPUT);
		break;
	   case 2:
		if((c>=0x81 && c<=0x9f) || (c>=0xe0 && c<=0xfc)) {
			c1 = c;	           
			stat = 1;
		} else {
			putc(ESC,OUTPUT);	/* put KANJI-OUT */
			putc(KO1,OUTPUT);
			putc(KO2,OUTPUT);
			ungetc(c,INPUT);
			stat = 0;
			}
		break;
	   case 3:				 /* hankaku kana */
		if((0xa1 <= c) && (c <= 0xdf)) {
			c &= 0x7f;
			putc(c,OUTPUT);
		}
		else {
			putc(KANAOUT,OUTPUT);
			ungetc(c,INPUT);
			stat = 0;
		}
		break;
	   }
	}
	if ( stat == 1 || stat == 2 ) {
			/* put KANJI-OUT */
			putc(ESC,OUTPUT);
			putc(KO1,OUTPUT);
			putc(KO2,OUTPUT);
	}
	else if (stat == 3) putc(KANAOUT,OUTPUT);
}

SJIS_JIS(sjis1,sjis2,jis1,jis2)
register unsigned sjis1,sjis2,*jis1,*jis2;
{
	if (sjis1 >= 0xe0)	/* The 2nd standard of JIS */
		sjis1 -=  0x40;

	if (sjis2 >= 0x9f) {
		*jis1 = (sjis1 - 0x88)*2 + 0x30;
		*jis2 = sjis2 - 0x7e;
	}
	else {
		if (sjis2 >= 0x7f)
			sjis2 -= 0x01;
		*jis1 = (sjis1 - 0x89)*2 + 0x31;
		*jis2 = sjis2 - 0x1f;
	}
}

jis_sjis(INPUT, OUTPUT)
FILE	*INPUT;
FILE	*OUTPUT;
{
	int stat;
	register unsigned int	c, c1, c2;
	unsigned int		s1, s2;
	static int		js78on = 0, js83on = 0;

	stat = 0;
	while ((c = getc(INPUT)) != EOF) {
	   switch (stat) {
	   case 0:
		if (c == ESC) 
		stat = 1;
		else if(c == KANAIN)
			stat = 6;
		else
			putc(c,OUTPUT);
		break;
	   case 1:			/*  ESC ----> ?  */
		switch (c) {
		case KO1:
			stat = 2;
			break;
		case KI1:
			stat = 3;
			break;
		case KANAIN:
			putc(ESC,OUTPUT);
			stat = 6;
			break;
		case ESC:
			putc(ESC,OUTPUT);
			break;
		default:
			putc(ESC,OUTPUT);
			putc(c,OUTPUT);
			stat = 0;
			break;
		}
		break;
	   case 2:			/* ESC --> KO1 --> ? */
		switch (c) {
		case KO2:		/* ^[(J : KOUT (to JIS_ROMAN) */
		case KO4:		/* ^[(B : KOUT (to ASCII) */
			if (js78on || js83on) {	/* JIS 78 or JIS 83 KANJI */
				js83on = 0;
				js78on = 0;
				stat = 0;
			}
			break;
		case KO3:		/* ^[(H : incorrect but used */
			if (js78on) {	/* JIS 78 KANJI */
				js78on = 0;
				stat = 0;
			}
			break;
		case KANAIN:
			putc(ESC,OUTPUT);
			putc(KO1,OUTPUT);
			stat = 6;
			break;
		case ESC:
			putc(ESC,OUTPUT);
			putc(KO1,OUTPUT);
			stat = 1;
			break;
		default:
			putc(ESC,OUTPUT);
			putc(KO1,OUTPUT);
			putc(c,OUTPUT);
			stat = 0;
			break;
		}
		break;
	   case 3:			/* ESC --> KI1 --> ? */
		switch (c) {
		case KI2:		/* ^[$@ : JIS 78 KIN */
			stat = 4;
			js78on = 1;
			break;
		case KI3:		/* ^[$B : JIS 83 KIN */
			stat = 4;
			js83on = 1;
			break;
		case KANAIN:
			putc(ESC,OUTPUT);
			putc(KI1,OUTPUT);
			stat = 6;
			break;
		case ESC:
			putc(ESC,OUTPUT);
			putc(KI1,OUTPUT);
			stat = 1;
			break;
		default:
			putc(ESC,OUTPUT);
			putc(KI1,OUTPUT);
			putc(c,OUTPUT);
			stat = 0;
			break;
		}
		break;
	   case 4:		/* JIS KANJI 1st */
		if (c == ESC)
			stat = 1;
		else if (c == KANAIN) ;
		else {
			c1 = c;
			stat = 5;
		}
		break;
	   case 5:		/* JIS KANJI 2nd */
		c2 = c;
		JIS_SJIS(c1,c2,&s1,&s2);
		putc(s1,OUTPUT);
		putc(s2,OUTPUT);
		stat = 4;
		break;
	   case 6:		/* HANKAKU KANA */
		if(c == KANAIN) ;
		else if(c == KANAOUT)
			stat = 0;
		else {
			if((c >=0x21) && (c <= 0x5f))
				c |= 0x80;
			putc(c,OUTPUT);
		}
		break;
	   }
	}
	return(0);
}

JIS_SJIS(jis1,jis2,sjis1,sjis2)
register unsigned jis1,jis2,*sjis1,*sjis2;
{
	if(jis1 >= 0x5f)    jis1 += 0x80;

	if((jis1 % 2) == 0) {
		*sjis1 = (jis1 - 0x30)/2 + 0x88;
		*sjis2 = jis2 + 0x7e;
	}
	else {
		if(jis2 >= 0x60)
			jis2 = jis2 + 0x01;
		*sjis1 = (jis1 - 0x31)/2 + 0x89;
		*sjis2 = jis2 + 0x1f;
	}
	*sjis1 &= 0xff;
	*sjis2 &= 0xff;
}
